package jeliot.gui;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;
import java.net.*;

import javax.swing.*; 
import javax.swing.event.*; 
import javax.swing.text.*;

import jeliot.parser.*;
import jeliot.theatre.*;
import jeliot.gui.*;

/**
  *
  * @author Pekka Uronen
  *
  * created         10.8.1999
  */
public class CodeEditor extends JComponent {

    private String template =
        "import jeliot.io.*;\n\n" +
        "public class MyClass {\n" +
	    "    public static void main() {\n\n" +
        "        // Your algorithm goes here.\n" +
	    "    }\n" +
        "}";

    JTextArea area = new JTextArea();
    {
        area.setMargin(new Insets(10, 10, 10, 10));
        area.setFont(new Font("Courier", Font.PLAIN, 12));
        area.setTabSize(4);
        clearProgram();
    }

    private JFileChooser fileChooser;
    private JFrame masterFrame;

    private ActionListener saver = new ActionListener() {
         public void actionPerformed(ActionEvent e) {
            saveProgram();
        }
    };
    private ActionListener loader = new ActionListener() {
         public void actionPerformed(ActionEvent e) {
            loadProgram();
        }
    };
    private ActionListener clearer = new ActionListener() {
         public void actionPerformed(ActionEvent e) {
            clearProgram();
        }
    };
    private ActionListener cutter = new ActionListener() {
         public void actionPerformed(ActionEvent e) {
            area.cut();
        }
    };
    private ActionListener copyist = new ActionListener() {
         public void actionPerformed(ActionEvent e) {
            area.copy();
        }
    };
    private ActionListener pasteur = new ActionListener() {
         public void actionPerformed(ActionEvent e) {
            area.paste();
        }
    };
    private ActionListener allSelector = new ActionListener() {
         public void actionPerformed(ActionEvent e) {
            area.selectAll();
        }
    };
    
    public void setMasterFrame(JFrame frame) {
        this.masterFrame = masterFrame;
    }
    
    public CodeEditor() {
        initFileChooser();
        setLayout(new BorderLayout());
        add("Center", new JScrollPane(area));
        add("North", makeToolBar());
    }

    private void initFileChooser() {
        // set up the file chooser with user's working
        // directory as default directory

        Properties prop = System.getProperties();
        String wdname = prop.getProperty("user.dir");
        File wd = new File(wdname);
        fileChooser = new JFileChooser();
        fileChooser.setCurrentDirectory(wd);
    }

    private JButton makeToolButton(String label, String iconName,
            ActionListener listener) {

        ImageIcon icon = new ImageIcon("images/"+iconName);
        JButton b = new JButton(label, icon);
        b.setVerticalTextPosition(AbstractButton.BOTTOM);
        b.setHorizontalTextPosition(AbstractButton.CENTER);
        b.setMargin(new Insets(0,0,0,0));
        b.addActionListener(listener);
        return b;
    }

    private JToolBar makeToolBar() {
        JButton loadButton = makeToolButton("Open", "openicon.gif", loader);
        JButton saveButton = makeToolButton("Save", "saveicon.gif", saver);
        JButton clearButton = makeToolButton("New", "newicon.gif", clearer);

        JButton cutButton = makeToolButton("Cut", "cuticon.gif", cutter);
        JButton copyButton = makeToolButton("Copy", "copyicon.gif", copyist);
        JButton pasteButton = makeToolButton("Paste", "pasteicon.gif", pasteur);
        
        JToolBar p = new JToolBar();
        p.add(clearButton);
        p.add(loadButton);
        p.add(saveButton);
        p.addSeparator();
        p.add(cutButton);
        p.add(copyButton);
        p.add(pasteButton);
        return p;
    }

    JMenu makeProgramMenu() {
        JMenu menu = new JMenu("Program");
        menu.setMnemonic(KeyEvent.VK_P);
        JMenuItem menuItem;

        menuItem = new JMenuItem("New");
        menuItem.setMnemonic(KeyEvent.VK_N);
        menuItem.setAccelerator(KeyStroke.getKeyStroke(
                KeyEvent.VK_N, ActionEvent.CTRL_MASK));
        menuItem.addActionListener(clearer);
        menu.add(menuItem);

        menuItem = new JMenuItem("Open", KeyEvent.VK_O);
        menuItem.setAccelerator(KeyStroke.getKeyStroke(
                KeyEvent.VK_O, ActionEvent.CTRL_MASK));
        menuItem.addActionListener(loader);
        menu.add(menuItem);
        
        menuItem = new JMenuItem("Save");
        menuItem.setMnemonic(KeyEvent.VK_S);
        menuItem.setAccelerator(KeyStroke.getKeyStroke(
                KeyEvent.VK_S, ActionEvent.CTRL_MASK));
        menuItem.addActionListener(saver);
        menu.add(menuItem);

        return menu;
    }

    JMenu makeEditMenu() {
        JMenu menu = new JMenu("Edit");
        menu.setMnemonic(KeyEvent.VK_E);
        JMenuItem menuItem;

        menuItem = new JMenuItem("Cut");
        menuItem.setMnemonic(KeyEvent.VK_T);
        menuItem.setAccelerator(KeyStroke.getKeyStroke(
                KeyEvent.VK_X, ActionEvent.CTRL_MASK));
        menuItem.addActionListener(cutter);
        menu.add(menuItem);

        menuItem = new JMenuItem("Copy");
        menuItem.setMnemonic(KeyEvent.VK_C);
        menuItem.setAccelerator(KeyStroke.getKeyStroke(
                KeyEvent.VK_C, ActionEvent.CTRL_MASK));
        menuItem.addActionListener(copyist);
        menu.add(menuItem);    

        menuItem = new JMenuItem("Paste");
        menuItem.setMnemonic(KeyEvent.VK_P);
        menuItem.setAccelerator(KeyStroke.getKeyStroke(
                KeyEvent.VK_V, ActionEvent.CTRL_MASK));
        menuItem.addActionListener(pasteur);
        menu.add(menuItem);

        menu.addSeparator();

        menuItem = new JMenuItem("Select All");
        menuItem.setMnemonic(KeyEvent.VK_A);
        menuItem.setAccelerator(KeyStroke.getKeyStroke(
                KeyEvent.VK_A, ActionEvent.CTRL_MASK));
        menuItem.addActionListener(allSelector);
        menu.add(menuItem);
        
        return menu;
    }
        
    void loadProgram() {
        fileChooser.rescanCurrentDirectory();
        int returnVal = fileChooser.showOpenDialog(masterFrame);
        if (returnVal == JFileChooser.APPROVE_OPTION) {
           File file = fileChooser.getSelectedFile();
           String program = readProgram(file);
           setProgram(program);
        }
    }

    void saveProgram() {
        fileChooser.rescanCurrentDirectory();
        int returnVal = fileChooser.showSaveDialog(masterFrame);
        if (returnVal == JFileChooser.APPROVE_OPTION) {
            File file = fileChooser.getSelectedFile();
            writeProgram(file);
        }   
    }

    void writeProgram(File file) {
        try {
            FileWriter w = new FileWriter(file);
            w.write(area.getText());
            w.close();
        }
        catch (IOException e) {
            e.printStackTrace();
        }
    } 

    String readProgram(File file) {
        try {
            Reader fr = new FileReader(file);
            BufferedReader r = new BufferedReader(fr);

            StringBuffer buff = new StringBuffer();
            String line;
            while ( (line = r.readLine()) != null ) {
                buff.append(line);
                buff.append("\n");
            }
            r.close();
            return buff.toString();
        }
        catch (IOException e) {
            e.printStackTrace();
            return null;
        }
    }

    void clearProgram() {
        area.setText(template);
    }

    void setProgram(String program) {
        area.setText(program);
    }

    public String getProgram() {
        return area.getText();
    }

    public void highlightStatement(final int left, final int right) {
        Runnable updateAComponent = new Runnable() {
            public void run() {
                area.select(left, right);
            }
        };
        SwingUtilities.invokeLater(updateAComponent);
    }

    public void highlight(int left, int right) {
        String program = getProgram();
        int a = Math.max(0, program.lastIndexOf('\n', left));
        int b = program.indexOf('\n', right);
        if (b == -1 ) {
            b = program.length() -1;
        }
        area.select(a, b);
    }

    
}
