package jeliot.lang;

import java.util.*;

import jeliot.parser.*;

/**
  * Instances of this class represent the types of Java. An instance of 
  * Type can be a primitive type, a reference type, or it can represent
  * null or void "types". A reference type may be a reference to a 
  * class or to an array. For each primitive type, there exists only 
  * one instance, which can be
  *
  * @author Pekka Uronen
  *
  * created         9.8.1999
  * revised         27.9.1999
  */
public abstract class Type {

    /** Returns an instance of Type corresponding to the given Class 
      * instance. 
      */
    public static Type getTypeByClass(Class c) {
        if (c == void.class) {
            return VoidType.voidTYPE;
        }
        if (c.isPrimitive()) {
            return PrimitiveType.getType(c);
        }
        if (c.isArray()) {
            return ArrayType.getType(c);
        }
        return ForeignType.getType(c);
    }

    /** Checks if a value of this type may be assigned to a variable of
      * given type t.
      */
    public abstract boolean mayBeAssignedTo(Type t);

    /** Checks if a value of this type can be cast to a value of type
      * t in an explicit cast. If 
      */
    public abstract boolean mayBeCastTo(Type t);
    
    /** Checks if this type is a numeric type. Only primitive types
      * other than boolean return true.
      */
    public boolean isNumeric() {
        return false;
    }
    
    /** Checks if this type is an integral type.
      */
    public boolean isIntegral() {
        return false;
    }
    
    public Value neutralValue() {
        return null;
    }
     
}
