package jeliot.parser;

import java.util.*;
import jeliot.parser.error.*;
import jeliot.lang.*;
import jeliot.theatre.*;
import jeliot.FeatureNotImplementedException;

/**
  *
  * @author Pekka Uronen
  *
  * created         30.9.1999
  */
public class PArrayAccess extends PExpression {
    
    private PExpression array;
    private PExpression index;
    private PName name;
    
    public PArrayAccess(PName name, PExpression index) {
        this(new PVariableAccessExpr(name), index);
        this.name = name;
        setLeftPos(name.getLeftPos());
    }
    
    public PArrayAccess(PExpression field, PExpression index) {
        this.array = field;
        this.index = index;
        array.setParent(this);
        index.setParent(this);
        setLeftPos(field.getLeftPos());
    }
    
    public String getIdentifier() {
        return name.getHead();
    }
    
    public void acceptVisitor(ProgramTreeVisitor visitor) {
        visitor.enterArrayAccess(this);
        if (array != null) {
            array.acceptVisitor(visitor);
        }
        if (index != null) {
            index.acceptVisitor(visitor);
        }
        visitor.leaveArrayAccess(this);
    }
    
    public void typeCheck() throws
            IllegalArrayAccessException,
            IllegalArrayIndexException {
                
        // check that the referred variable's type is array.
        Type atype = array.getType();
        if (! (atype instanceof ArrayType) ) {
            throw new IllegalArrayAccessException(atype, this);
        }
        // check that the index type is integral
        Type itype = index.getType();
        if (!itype.isIntegral() || itype == PrimitiveType.longTYPE) {
            throw new IllegalArrayIndexException(itype, this);
        }
        setType( ((ArrayType)atype).getComponentType() );
    }
    
    public Value evaluate(Director director) {
        super.evaluate(director);
        Value arrayVal = array.evaluate(director);
        ArrayInstance ar = (ArrayInstance)arrayVal.getValue();
        
        Value indexVal = index.evaluate(director);
        int i = ((Integer)indexVal.getValue()).intValue();
        
        try {
            VariableInArray vara = ar.getVariableAt(i);
            Value retval = (Value)vara.getValue().clone();
            director.showArrayAccess(vara, indexVal, retval);
            return retval;
        }
        catch (ArrayIndexOutOfBoundsException e) {
            e.printStackTrace();   
        }
        return null;
    }
    
    public Variable evaluateVariable(Director director) {
        super.evaluate(director);
        Value arrayVal = array.evaluate(director);
        ArrayInstance ar = (ArrayInstance)arrayVal.getValue();
        
        Value indexVal = index.evaluate(director);
        int i = ((Integer)indexVal.getValue()).intValue();
        
        try {
            VariableInArray vara = ar.getVariableAt(i);
            director.showArrayVariableAccess(vara, indexVal);
            return vara;
        }
        catch (ArrayIndexOutOfBoundsException e) {
            e.printStackTrace();   
        }
        return null;
    }
    
}