package jeliot.parser;

import java.util.*;
import jeliot.lang.*;
import jeliot.theatre.*;

/**
  * PDoWhile represents a do-while statement.
  *
  * @author Pekka Uronen
  *
  * created         6.8.1999
  */
public class PDoWhile extends PStatement {

    private PStatement statement;
    private PExpression check;
    private PStatement doStatement;

    /** Starting position of the word "do". */
    int doLeft;

    protected PDoWhile(PStatement statement, PExpression check,
            int doLeft, int left, int right) {

        super(left, right);
        this.check = check;
        this.statement = statement;
        this.doLeft = doLeft;
        this.doStatement = new PStatement(doLeft, doLeft+2);

        check.setParent(this);
		statement.setParent(this);
    }

    public void acceptVisitor(ProgramTreeVisitor visitor) {
        visitor.enterDoWhile(this);
        check.acceptVisitor(visitor);
        statement.acceptVisitor(visitor);
        visitor.leaveDoWhile(this);
    }

    void execute(Director director) throws Return {
        // inform director we are at a new statement

        //director.atStatement(this);
        director.enterLoop(this);

        boolean contLoop;
        do {
            // execute contained statement
            statement.execute(director);

            // back to this statement
            director.atStatement(this);
            Scratch scratch = director.openScratch();

            // evaluate the expression
            Value v = check.evaluate(director);
            contLoop = ((Boolean)v.getValue()).booleanValue();
            director.atStatement(this);
            if ( contLoop ) {
                director.continueLoop(this, v);
            }
            else {
                director.exitLoop(this, v);
            }
            director.closeScratch(scratch);
        } while (contLoop);
    }

    public PExpression getCheck() {
        return check;
    }

    public String statementName() {
        return "do-while";
    }

}
