package jeliot.script.jeliot.io;

import jeliot.lang.*;
import jeliot.theatre.*;

/**
  * @author Pekka Uronen
  *
  * created         4.10.1999
  */
public class Input {

    public static Animator readInt() {
        return new InputAnimator(
            "Please input an integer.",
            new InputValidator() {
                public void validate(String s) {
                    try {
                        Integer i = new Integer(s);
                        accept(new Value(PrimitiveType.intTYPE, i));
                    }
                    catch (NumberFormatException e) { }
                }
            }
        );
    }
    
    public static Animator readDouble() {
        return new InputAnimator(
            "Please input a double.",
            new InputValidator() {
                public void validate(String s) {
                    try {
                        Double d = new Double(s);
                        accept(new Value(PrimitiveType.doubleTYPE, d));
                    }
                    catch (NumberFormatException e) { }
                }
            }
        );
    }
    
    private static class InputAnimator extends Animator {    

        private String prompt;
        private InputValidator validator;
        
        private InputAnimator(String prompt, InputValidator valid) {
            this.prompt = prompt;
            this.validator = valid;
        }
        
        public void animate(Director director) {
            Value val = director.getInput(prompt, validator);
            setReturnValue(val);
        }
        
    }
        
}
