package jeliot.theatre;

import java.awt.*;
import java.lang.reflect.*;
import jeliot.parser.*;
import jeliot.lang.*;
import jeliot.gui.*;
import jeliot.*;

/**
  * @author Pekka Uronen
  *
  * created         25.8.1999
  * revised         4.10.1999
  */
public class DefaultAnimator extends Animator {
    
    private ForeignMethodPointer fmp;
    private Value[] args;
    
    public DefaultAnimator(ForeignMethodPointer fmp, Value[] args) {
        this.fmp = fmp;
        this.args = args;
    }
    
    public void animate(Director director) { 
        // Get necessary objects from the director.
        Theatre theatre = director.getTheatre();
        ActorFactory factory = director.getFactory();
        AnimationEngine engine = director.getEngine();
        Scratch scratch = director.getCurrentScratch();
        
        // Create the actor for the invocation.
        int n = args.length;
        SMIActor actor = factory.produceSMIActor(fmp, n);
        ExpressionActor expr = scratch.getExpression(1);
        
        Point invop = expr.getRootLocation();
        actor.setLocation(invop);
        
        // Create actors and reserve places for all argument values,
        // and create animations to bring them in their right places.
        ValueActor[] argact = new ValueActor[n];
        Animation[] fly = new Animation[n];
        for (int i = 0; i < n; ++i) {
            argact[i] = args[i].getActor();
            fly[i] = argact[i].fly(actor.reserve(argact[i]));
        }
        // Calculate the size of the invocation actor, taking in count
        // the argument actors. 
        actor.calculateSize();
        
        // Prepare the theatre.
        theatre.capture();
        
        // Introduce the invocation.
        engine.showAnimation(actor.appear(invop));
        
        // Bring in arguments and bind them to the invocation actor.
        engine.showAnimation(fly);
        for (int i = 0; i < n; ++i) {
            actor.bind(argact[i]);
        }
        
        // Invoke the method to get the real result.
        Value result = fmp.invokeStatic(args);
        setReturnValue(result);
        
        // Create actor for the result.
        ValueActor resultAct = factory.produceValueActor(result);
        result.setActor(resultAct);
        
        // Create a bubble for the result value.
        BubbleActor bubble = factory.produceBubble(actor);
        Point p = new Point(actor.getX() + actor.getWidth()/2,
                actor.getY() + actor.getHeight() + 25);
        bubble.setLocation(p);
        bubble.setActor(resultAct);
        engine.showAnimation(bubble.appear(p));
        
        // remove the invocation actor
        theatre.removePassive(actor);
        bubble.removeTip();
        
        // Move the bubble to its place on scratch.
        Point bubbleLoc = expr.reserve(bubble);
        engine.showAnimation(bubble.fly(bubbleLoc));
        expr.bind(bubble);
        
        // De-activate the theatre.
        theatre.release();
    }
    
}
