package jeliot.theatre;

import java.awt.*;
import javax.swing.*;
import jeliot.parser.*;
import jeliot.lang.*;
import jeliot.gui.*;

/**
  * Writes scripts that visualize various expressions.
  *
  * @author Pekka Uronen
  *
  * created         9.8.1999
  */
public class ScriptWriter {

    ActorFactory factory;
    Theatre theatre;
    TheatreManager manager;
    AnimationEngine engine;
    
    public ScriptWriter(AnimationEngine engine, Theatre theatre,
            ActorFactory factory) {

        this.engine = engine;
        this.theatre = theatre;
        this.manager = theatre.getManager();
        this.factory = factory;
    }

    public void addStage(Stage stage) {
        theatre.capture();
        Point sLoc = manager.reserve(stage);
        engine.showAnimation(stage.appear(sLoc));
        manager.bind(stage);
        theatre.release();
    }

    public void introduceValue(ValueActor actor, Point loc) {
        theatre.capture();
        engine.showAnimation(actor.appear(loc));
        theatre.release();
    }
    
    public void showBinaryExpression(
            Actor fAct,
            OperatorActor opAct,
            Actor sAct,
            OperatorActor eqAct,
            Actor rAct,
            Scratch scratch) {

        theatre.capture();
        ExpressionActor exp = scratch.getExpression(5);
        Point fLoc = exp.reserve(fAct);
        Point oLoc = exp.reserve(opAct);
        Point sLoc = exp.reserve(sAct);
        Point eLoc = exp.reserve(eqAct);
        Point rLoc = exp.reserve(rAct);

        engine.showAnimation(
            new Animation[] {
                fAct.fly(fLoc),
                sAct.fly(sLoc)
            });

        exp.bind(fAct);
        exp.bind(sAct);
        theatre.updateCapture();
        
        engine.showAnimation(opAct.appear(oLoc));
        exp.bind(opAct);
        theatre.updateCapture();
        engine.showAnimation(eqAct.appear(eLoc));
        exp.bind(eqAct);
        theatre.updateCapture();
        engine.showAnimation(rAct.appear(rLoc));
        exp.bind(rAct);
        exp.setLight(Actor.SHADED);
        
        theatre.release();
    }
        
    public void showLocalVariableDeclaration(
            VariableActor actor,
            Stage stage) {
        theatre.capture();
        Point loc = stage.reserve(actor);
        engine.showAnimation(actor.appear(loc));
        stage.bind();
        theatre.release();
    }

    public void showAssignment(
            VariableActor vrAct,
            ValueActor vlAct,
            ValueActor ctAct) {

        theatre.capture();
        Point dest = vrAct.reserve(ctAct);
        engine.showAnimation(vlAct.fly(dest));
        vrAct.bind();
        theatre.removeActor(vlAct);
        theatre.release();
    }

    public void showMessage(MessageActor message, Actor anchor) {
        Point aloc = anchor.getRootLocation();
        Dimension asize = anchor.getSize();
        aloc.translate(asize.width/2, 0);
        Dimension msize = message.getSize();
        Dimension tsize = theatre.getSize();
        if (aloc.x - msize.width/2 < 10) {
            aloc.x = 10;
        }
        else if (aloc.x + msize.width/2 > tsize.width-10) {
            aloc.x = tsize.width-10-msize.width;
        }
        else {
            aloc.x -= msize.width/2;
        }
        aloc.y += asize.height + 10;
        showMessage(message, new Point(aloc.x, aloc.y));
        theatre.removeActor(anchor);
    }

    public void showMessage(MessageActor message) {
        Dimension msize = message.getSize();
        Dimension tsize = theatre.getSize();
        int x = (tsize.width-msize.width)/2;
        int y = (tsize.height-msize.height)/2;
        showMessage(message, new Point(x, y));
    }

    public void showMessage(MessageActor message, Point p) {
        theatre.capture();
        engine.showAnimation(message.appear(p));
        theatre.removeActor(message);
        theatre.release();
    }

}
